==> fetch.php?media=resources_multi_hardware:lorad_1.0.5.tar.gz: lorad_1.0.5/core_v1/beacon.c <==
// Copyright (c) 2018 Kerlink SA. All rights reserved.

#include "beacon.h"

#include <stdint.h>
#include <string.h>
#include <sys/types.h>

#include "utils/logger.h"

#include "clock.h"
#include "config.h"

static int build_beacon(struct core *core, struct lgw_pkt_tx_s *beacon)
{
	struct config_beacon *config_beacon = &core->config->core->beacon;
	struct timespec current_tp, beacon_tp;
	size_t rfu1_size, rfu2_size;
	uint32_t channel = 0;
	uint8_t *payload;
	uint16_t crc;
	int gps;

	memset(beacon, 0, sizeof(*beacon));

	// Compute beacon time
	if (clock_get_gps_time(core->clock->tai_offset, &current_tp) < 0) {
		error("Failed to get current gps time");
		return -1;
	}

	beacon_tp.tv_sec = current_tp.tv_sec + (config_beacon->period - (current_tp.tv_sec % config_beacon->period));
	beacon_tp.tv_nsec = 0;

	if (clock_get_counter_from_gps_time(core, &beacon_tp, &beacon->count_us) < 0) {
		error("Failed to get next beacon counter");
		return -1;
	}

	// Beacon radio configuration
	beacon->tx_mode = ON_GPS;
	beacon->rf_chain = 0;

	debug("Requested beacon power: %i dB", config_beacon->power);
	beacon->rf_power = config_beacon->power - core->config->core->antenna_gain + core->config->core->insertion_loss;
	debug("Selected beacon power: %i dB", beacon->rf_power);

	beacon->modulation = MOD_LORA;
	beacon->bandwidth = config_beacon->bandwidth;
	beacon->datarate = config_beacon->spreading_factor;
	beacon->coderate = CR_LORA_4_5;
	beacon->invert_pol = false;
	beacon->preamble = 10;
	beacon->no_crc = true;
	beacon->no_header = true;

	switch (beacon->datarate) {
	case DR_LORA_SF8:
		rfu1_size = 1;
		rfu2_size = 3;
		break;
	case DR_LORA_SF9:
		rfu1_size = 2;
		rfu2_size = 0;
		break;
	case DR_LORA_SF10:
		rfu1_size = 3;
		rfu2_size = 1;
		break;
	case DR_LORA_SF12:
		rfu1_size = 5;
		rfu2_size = 3;
		break;
	default:
		error("Invalid beacon datarate");
		return -1;
	}

	if (config_beacon->frequencies > 1) {
		channel = (beacon_tp.tv_sec / config_beacon->period) % config_beacon->frequencies;
	}

	beacon->freq_hz = config_beacon->frequency + (channel * config_beacon->frequency_step);

	// Beacon data
	beacon->size = BEACON_SIZE(rfu1_size, rfu2_size);

	payload = BEACON_TIME(beacon->payload, rfu1_size, rfu2_size);
	payload[0] = (beacon_tp.tv_sec & 0x000000FF) >> 0;
	payload[1] = (beacon_tp.tv_sec & 0x0000FF00) >> 8;
	payload[2] = (beacon_tp.tv_sec & 0x00FF0000) >> 16;
	payload[3] = (beacon_tp.tv_sec & 0xFF000000) >> 24;

	crc = beacon_crc(BEACON_COMMON(beacon->payload, rfu1_size, rfu2_size), BEACON_COMMON_DATA_SIZE(rfu1_size, rfu2_size));
	payload = BEACON_CRC1(beacon->payload, rfu1_size, rfu2_size);
	payload[0] = (crc & 0x00FF) >> 0;
	payload[1] = (crc & 0xFF00) >> 8;

	payload = BEACON_INFODESC(beacon->payload, rfu1_size, rfu2_size);
	payload[0] = config_beacon->infodesc;

	gps = beacon_double_to_integer(config_beacon->latitude, 90.0, 24);
	payload = BEACON_LATITUDE(beacon->payload, rfu1_size, rfu2_size);
	payload[0] = (gps & 0x0000FF) >> 0;
	payload[1] = (gps & 0x00FF00) >> 8;
	payload[2] = (gps & 0xFF0000) >> 16;

	gps = beacon_double_to_integer(config_beacon->longitude, 180.0, 24);
	payload = BEACON_LONGITUDE(beacon->payload, rfu1_size, rfu2_size);
	payload[0] = (gps & 0x0000FF) >> 0;
	payload[1] = (gps & 0x00FF00) >> 8;
	payload[2] = (gps & 0xFF0000) >> 16;

	crc = beacon_crc(BEACON_SPECIFIC(beacon->payload, rfu1_size, rfu2_size), BEACON_SPECIFIC_DATA_SIZE(rfu1_size, rfu2_size));
	payload = BEACON_CRC2(beacon->payload, rfu1_size, rfu2_size);
	payload[0] = (crc & 0x00FF) >> 0;
	payload[1] = (crc & 0xFF00) >> 8;

	return 0;
}

int beacon_core_schedule(struct core *core, struct jit_queue_s *jit)
{
	struct lgw_pkt_tx_s beacon;

	if (jit->num_beacon > 0) {
		return 0;

	} else if (!core->clock->core->tai.sync) {
		error("Cannot schedule beacon: GPS time is not synchronized");
		return 0;

	} else if (build_beacon(core, &beacon) < 0) {
		error("Failed to build beacon");
		return -1;

	} else if (jit_enqueue(core, jit, &beacon, JIT_PKT_TYPE_BEACON) != JIT_ERROR_OK) {
		error("Failed to enqueue beacon");
		return -1;
	}

	info("The next beacon has been scheduled");

	return 0;
}

==> fetch.php?media=resources_multi_hardware:lorad_1.2.0.tar.gz: lorad_1.2.0/core_v1/beacon.c <==
// Copyright (c) 2018 Kerlink SA. All rights reserved.

#include "beacon.h"

#include <stdint.h>
#include <string.h>
#include <sys/types.h>

#include "lib/loragw/hal.h"

#include "utils/logger.h"

#include "clock.h"
#include "config.h"

static int build_beacon(struct core *core, struct lgw_pkt_tx_s *beacon)
{
	struct config_beacon *config_beacon = &core->config->beacon;
	static uint8_t saved_infodesc = BEACON_INFODESC_MAX;
	struct timespec current_tp, beacon_tp;
	size_t rfu1_size, rfu2_size;
	uint32_t channel = 0;
	uint8_t *payload;
	uint8_t infodesc;
	uint16_t crc;
	size_t n;
	int gps;

	memset(beacon, 0, sizeof(*beacon));

	// Compute beacon time
	if (clock_get_gps_time(core->clock->tai_offset, &current_tp) < 0) {
		error("Failed to get current gps time");
		return -1;
	}

	beacon_tp.tv_sec = current_tp.tv_sec + (config_beacon->period - (current_tp.tv_sec % config_beacon->period));
	beacon_tp.tv_nsec = 0;

	if (clock_get_counter_from_gps_time(core, &beacon_tp, &beacon->count_us) < 0) {
		error("Failed to get next beacon counter");
		return -1;
	}

	// Beacon radio configuration
	beacon->tx_mode = ON_GPS;
	beacon->rf_chain = 0;

	debug("Requested beacon power: %i dB", config_beacon->power);
	beacon->rf_power = config_beacon->power - core->config->core->antenna_gain + core->config->core->insertion_loss;
	debug("Selected beacon power: %i dB", beacon->rf_power);

	beacon->modulation = MOD_LORA;

	switch (config_beacon->bandwidth) {
	case 500000:
		beacon->bandwidth = BW_500KHZ;
		break;
	default:
		beacon->bandwidth = BW_125KHZ;
		break;
	}

	switch (config_beacon->spreading_factor) {
	case 8:
		beacon->datarate = DR_LORA_SF8;
		break;
	case 10:
		beacon->datarate = DR_LORA_SF10;
		break;
	case 12:
		beacon->datarate = DR_LORA_SF12;
		break;
	default:
		beacon->datarate = DR_LORA_SF9;
		break;
	}

	beacon->coderate = CR_LORA_4_5;
	beacon->invert_pol = false;
	beacon->preamble = 10;
	beacon->no_crc = true;
	beacon->no_header = true;

	switch (beacon->datarate) {
	case DR_LORA_SF8:
		rfu1_size = 1;
		rfu2_size = 3;
		break;
	case DR_LORA_SF10:
		rfu1_size = 3;
		rfu2_size = 1;
		break;
	case DR_LORA_SF12:
		rfu1_size = 5;
		rfu2_size = 3;
		break;
	default:
		rfu1_size = 2;
		rfu2_size = 0;
		break;
	}

	if (config_beacon->frequencies > 1) {
		channel = (beacon_tp.tv_sec / config_beacon->period) % config_beacon->frequencies;
	}

	beacon->freq_hz = config_beacon->frequency + (channel * config_beacon->frequency_step);

	// Beacon data
	beacon->size = BEACON_SIZE(rfu1_size, rfu2_size);

	payload = BEACON_TIME(beacon->payload, rfu1_size, rfu2_size);
	payload[0] = (beacon_tp.tv_sec & 0x000000FF) >> 0;
	payload[1] = (beacon_tp.tv_sec & 0x0000FF00) >> 8;
	payload[2] = (beacon_tp.tv_sec & 0x00FF0000) >> 16;
	payload[3] = (beacon_tp.tv_sec & 0xFF000000) >> 24;

	crc = beacon_crc(BEACON_COMMON(beacon->payload, rfu1_size, rfu2_size), BEACON_COMMON_DATA_SIZE(rfu1_size, rfu2_size));
	payload = BEACON_CRC1(beacon->payload, rfu1_size, rfu2_size);
	payload[0] = (crc & 0x00FF) >> 0;
	payload[1] = (crc & 0xFF00) >> 8;

	for (n = 0, infodesc = saved_infodesc; n <= BEACON_INFODESC_MAX; n++) {
		if (infodesc < BEACON_INFODESC_MAX) {
			infodesc++;
		} else {
			infodesc = 0;
		}

		if (infodesc < BEACON_INFODESC_NETID &&
		    infodesc < config_beacon->nb_coordinates) {
			break;
		} else if (infodesc == BEACON_INFODESC_NETID &&
			   config_beacon->has_netid) {
			break;
		}
	}

	if (n <= BEACON_INFODESC_MAX) {
		saved_infodesc = infodesc;

		payload = BEACON_INFODESC(beacon->payload, rfu1_size, rfu2_size);
		payload[0] = infodesc;

		if (infodesc < BEACON_INFODESC_NETID) {
			gps = beacon_double_to_integer(config_beacon->coordinates[infodesc].latitude, 90.0, 24);
			payload = BEACON_LATITUDE_NETID(beacon->payload, rfu1_size, rfu2_size);
			payload[0] = (gps & 0x0000FF) >> 0;
			payload[1] = (gps & 0x00FF00) >> 8;
			payload[2] = (gps & 0xFF0000) >> 16;

			gps = beacon_double_to_integer(config_beacon->coordinates[infodesc].longitude, 180.0, 24);
			payload = BEACON_LONGITUDE_GWID(beacon->payload, rfu1_size, rfu2_size);
			payload[0] = (gps & 0x0000FF) >> 0;
			payload[1] = (gps & 0x00FF00) >> 8;
			payload[2] = (gps & 0xFF0000) >> 16;

		} else if (infodesc == BEACON_INFODESC_NETID) {
			payload = BEACON_LATITUDE_NETID(beacon->payload, rfu1_size, rfu2_size);
			payload[0] = (config_beacon->netid & 0x0000FF) >> 0;
			payload[1] = (config_beacon->netid & 0x00FF00) >> 8;
			payload[2] = (config_beacon->netid & 0xFF0000) >> 16;

			payload = BEACON_LONGITUDE_GWID(beacon->payload, rfu1_size, rfu2_size);
			payload[0] = (config_beacon->gwid & 0x0000FF) >> 0;
			payload[1] = (config_beacon->gwid & 0x00FF00) >> 8;
			payload[2] = (config_beacon->gwid & 0xFF0000) >> 16;
		}
	}

	crc = beacon_crc(BEACON_SPECIFIC(beacon->payload, rfu1_size, rfu2_size), BEACON_SPECIFIC_DATA_SIZE(rfu1_size, rfu2_size));
	payload = BEACON_CRC2(beacon->payload, rfu1_size, rfu2_size);
	payload[0] = (crc & 0x00FF) >> 0;
	payload[1] = (crc & 0xFF00) >> 8;

	return 0;
}

int beacon_core_schedule(struct core *core, struct jit_queue_s *jit)
{
	struct lgw_pkt_tx_s beacon;

	if (jit->num_beacon > 0) {
		return 0;

	} else if (!core->clock->core->tai.sync) {
		error("Cannot schedule beacon: GPS time is not synchronized");
		return 0;

	} else if (build_beacon(core, &beacon) < 0) {
		error("Failed to build beacon");
		return -1;

	} else if (jit_enqueue(core, jit, &beacon, JIT_PKT_TYPE_BEACON) != JIT_ERROR_OK) {
		error("Failed to enqueue beacon");
		return -1;
	}

	info("The next beacon has been scheduled");

	return 0;
}
